/*
pbgm.c  Version 0.3.0  Powerball Drawings
Copyright (C) 2002-2010   dondalah721@yahoo.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	int pwrball;
    	double odds;
    	double getpb(int select, int pwrball);
    	odds = getpb(select,pwrball);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in pbprob.c regarding the */
/* calls to getpb.c. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "rnd.h"

#define CENTURY 10436
#define PWRBALL 42
#define WHITEBALL 53
#define FIVE 5

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s ticket\n", pgm);
   fprintf(stderr,"Where ticket is 01 02 03 04 05 06\n");
   fprintf(stderr,"Numbers 01 02 03 04 05 are 1 - 43\n");
   fprintf(stderr,"Number  06 is 1 - 42  (Powerball)\n");
   fprintf(stderr,"To see probabilities, run pbprob\n");
   fprintf(stderr,"To test quick picks,  run qpick\n");
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,maxwhball,indx;
   unsigned char *p,*q,*r;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char whiteball[64];
   unsigned char ticket[64];
   unsigned char game[16];
   unsigned char trial[16];
   int tmp;
   int ticketbb;
   int winpb;
   int totmtch,won;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning,totprize,netearn;
   double *prizeptr;
   double prize[16];
   double odds,sum,totbingo,chisq;
   double totprob;
   double expected;
   double getpb(int select, int pwrball);
   double *bingoptr;
   double bingo[16];
   p = (unsigned char *) ticket;
   q = (unsigned char *) ticket + WHITEBALL;
   while (p < q)
      *p++ = (unsigned char) '\0';
   if (argc != 7) putstx(*argv);
   gameptr = (unsigned char *) game;
   i = 1;
   while (i <= 5)
      {
      indx = atoi(*(argv+i));
      if (indx < 1 || indx > WHITEBALL)
	 {
	 fprintf(stderr,"pbgm: ticket number %s "
	    "is invalid\n",
	    *(argv+i));
	 putstx(*argv);
	 } /* if invalid number */
      p = (unsigned char *) ticket + indx;
      if (*p != 255)
         *p = (unsigned char) 255;
      else
	 {
	 fprintf(stderr,"pbgm: ticket number %s "
	    "is a duplicate\n",
	    *(argv+i));
	 putstx(*argv);
	 } /* duplicate number */
      *gameptr++ = (unsigned char) indx;
      i++;
      } /* for each number on ticket */
   ticketbb = atoi(*(argv+i));
   if (ticketbb < 1 || ticketbb > PWRBALL)
      {
      fprintf(stderr,"pbgm: powerball number %s "
         "is invalid\n",
         *(argv+i));
      putstx(*argv);
      } /* if invalid powerball */
   p = (unsigned char *) ticket + ticketbb;
   if (*p == 255)
      {
      fprintf(stderr,"pbgm: powerball number %s "
         "is a duplicate\n",
         *(argv+i));
      putstx(*argv);
      } /* duplicate number */
   /* one game for $1.00 */
   /* twice/week for 100 years */
   /* 52 * 2 * 100 = 10400 */
   /* Based on Julian days, it computes to 10436 */
   /* ((2488434.5 - 2451910.5) / 7) * 2 = 10436 */
   maxplay = CENTURY;
   sd = (unsigned char *) rndinit();
   prizeptr = (double *) prize;
   *prizeptr++ = 5000000.00;
   *prizeptr++ = 100000.00;
   *prizeptr++ = 5000.00;
   *prizeptr++ = 100.00;
   *prizeptr++ = 100.00;
   *prizeptr++ = 7.00;
   *prizeptr++ = 7.00;
   *prizeptr++ = 4.00;
   *prizeptr   = 3.00;
   printf("Powerball ticket:  ");
   gameptr = (unsigned char *) game;
   i = FIVE;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* first 5 numbers on ticket */
   printf("- %2d\n\n", ticketbb);
   printf("Match      Wins          Prize    "
      "   Expected           Odds\n");
   bingoptr = (double *) bingo;
   i = 9;
   while (i--) *bingoptr++ = 0.0;
   bingoptr = (double *) bingo;
   totplay = maxplay;
   while (totplay--)
      {
      p = (unsigned char *) whiteball;
      q = (unsigned char *) whiteball + WHITEBALL;
      i = 0;
      while (p < q)
         *p++ = (unsigned char) ++i;
      *p = (unsigned char) '\0';
      totmtch = winpb = 0;
      p = (unsigned char *) whiteball;
      q = (unsigned char *) whiteball + WHITEBALL - 1;
      r = (unsigned char *) ticket;
      trialptr = (unsigned char *) trial;
      maxwhball = WHITEBALL;
      i = FIVE;
      while (i--)
         {
         indx = rndnum(maxwhball,sd);
         tmp = (int) *(p+indx);
         *trialptr++ = (unsigned char) tmp;
         if (*(r+tmp) == 255) totmtch++;
         *(p+indx) = *q;
         *q-- = '\0';
         maxwhball--;
         } /* pick 5 balls without replacement */
      winpb = rndnum(PWRBALL,sd) + 1;
      if (winpb != ticketbb) winpb = 0;
      won = 999999;
      if (totmtch == FIVE)
         {
         if (winpb) won = 0;
         else won = 1;
         } /* select 5 */
      else if (totmtch == 4)
         {
         if (winpb) won = 2;
         else won = 3;
         } /* select 4 */
      else if (totmtch == 3)
         {
         if (winpb) won = 4;
         else won = 5;
         } /* select 3 */
      else if (totmtch == 2 && winpb)
         won = 6;
      else if (totmtch == 1 && winpb)
         won = 7;
      else if (totmtch == 0 && winpb)
         won = 8;
      else won = 9;
      *(bingoptr+won) += 1.0;
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   dblplay = (double) maxplay;
   totprize = 0.0;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 54979155.0; */
   prob = getpb(5,1);
   expected = dblplay / prob;
   printf("5 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 1249527.0; */
   prob = getpb(5,0);
   expected = dblplay / prob;
   printf("5 0   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 274896.0; */
   prob = getpb(4,1);
   expected = dblplay / prob;
   printf("4 1   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 6248.0; */
   prob = getpb(4,0);
   expected = dblplay / prob;
   printf("4 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 7049.0; */
   prob = getpb(3,1);
   expected = dblplay / prob;
   printf("3 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 161.0; */
   prob = getpb(3,0);
   expected = dblplay / prob;
   printf("3 0   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 557.0; */
   prob = getpb(2,1);
   expected = dblplay / prob;
   printf("2 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 121.0; */
   prob = getpb(1,1);
   expected = dblplay / prob;
   printf("1 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   if (*bingoptr > 0.0)
      {
      winning = (*prizeptr) * (*bingoptr);
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   /* prob = 84.0; */
   prob = getpb(0,1);
   expected = dblplay / prob;
   printf("0 1   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   bingoptr = (double *) bingo;
   sum = totbingo = totprob = 0.0;
   odds = getpb(5,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(5,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(4,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(4,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(3,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(3,0);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(2,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(1,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getpb(0,1);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;

   sum += ((dblplay - totbingo)
      * (dblplay - totbingo)
      / (1.0 - totprob));

   expected = dblplay / (1.0 / (1.0 - totprob));
   printf("0 0   %9.0f  %13.2f  %13.2f   %21.10f\n",
      (dblplay - totbingo),
      0.0,
      expected,
      (1.0 / (1.0 - totprob)));
   printf("        -------      ---------\n");
   printf("Total %9.0f  %13.2f\n\n",
      dblplay, totprize);
   printf("Total Cost       %13.2f\n\n",
      dblplay);
   netearn = totprize - dblplay;
   printf("Net Income       %13.2f\n\n",
      netearn);
   chisq = (sum / dblplay) - dblplay;
   printf("Chi-square %f\n", chisq);
   printf("Range at 95 percent "
      "is 2.70039 to 19.0228\n");
   free(sd);
   return(0);
   } /* main */
